/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.oozie.action.hadoop;

import static org.apache.oozie.action.hadoop.HiveMain.HIVE_DEFAULT_CONF;
import static org.apache.oozie.action.hadoop.HiveMain.HIVE_EXEC_L4J_PROPS;
import static org.apache.oozie.action.hadoop.HiveMain.HIVE_L4J_PROPS;
import static org.apache.oozie.action.hadoop.HiveMain.HIVE_SITE_CONF;
import static org.apache.oozie.action.hadoop.LauncherMapper.CONF_OOZIE_ACTION_MAIN_CLASS;

import java.io.IOException;
import java.io.StringReader;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.oozie.action.ActionExecutorException;
import org.apache.oozie.client.WorkflowAction;
import org.apache.oozie.util.XConfiguration;
import org.apache.oozie.util.XmlUtils;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;

public class HiveActionExecutor extends JavaActionExecutor {

    public HiveActionExecutor() {
        super("hive");
    }

    @Override
    protected List<Class> getLauncherClasses() {
        List<Class> classes = super.getLauncherClasses();
        classes.add(LauncherMain.class);
        classes.add(MapReduceMain.class);
        classes.add(HiveMain.class);
        return classes;
    }

    @Override
    protected String getLauncherMain(Configuration launcherConf, Element actionXml) {
        return launcherConf.get(CONF_OOZIE_ACTION_MAIN_CLASS, HiveMain.class.getName());
    }

    @Override
    void injectActionCallback(Context context, Configuration launcherConf) {
    }

    protected void addResourceToCache(Configuration conf, FileSystem fs, Path resourcePath)
    throws ActionExecutorException {
        try {
            if (!fs.exists(resourcePath) || !fs.isFile(resourcePath)) {
                return;
            }
            Path parentPath = resourcePath.getParent();
            String resourceName = resourcePath.getName();
            addToCache(conf, parentPath, resourceName + "#" + resourceName, false);
        } catch (Exception ex) {
            throw convertException(ex);
        }
    }

    @Override
    protected Configuration setupLauncherConf(Configuration conf, Element actionXml, Path appPath, Context context)
            throws ActionExecutorException {
        try {
            super.setupLauncherConf(conf, actionXml, appPath, context);
            Namespace ns = actionXml.getNamespace();

            // Look for hive-log4j.properties and hive-exec-log4j.properties
            // in the configuration directory and add them to the distributed
            // cache.
            Element e = actionXml.getChild("conf-dir", ns);
            if (e != null) {
                Path hiveConfPath = new Path(appPath, e.getTextTrim());
                FileSystem fs = getActionFileSystem(context, actionXml);
                FileStatus status = fs.getFileStatus(hiveConfPath);
                if (status.isDir()) {
                    addResourceToCache(conf, fs, new Path(hiveConfPath,HIVE_L4J_PROPS));
                    addResourceToCache(conf, fs, new Path(hiveConfPath, HIVE_EXEC_L4J_PROPS));
                }
            }

            // Make the query file available to Hive.
            String script = actionXml.getChild("script", ns).getTextTrim();
            String scriptName = new Path(script).getName();
            addToCache(conf, appPath, script + "#" + scriptName, false);

            return conf;
        } catch (IOException ex) {
            throw convertException(ex);
        }
    }

    @Override
    @SuppressWarnings("unchecked")
    Configuration setupActionConf(Configuration actionConf, Context context,
            Element actionXml, Path appPath) throws ActionExecutorException {

        try {
            // Read hive-default.xml, hive-site.xml, and any configuration
            // properties defined in the workflow and overlay these properties
            // onto the action configuration in this order.
            Namespace ns = actionXml.getNamespace();
            Element e = actionXml.getChild("conf-dir", ns);
            if (e != null) {
                Path hiveConfPath = new Path(appPath, e.getTextTrim());
                FileSystem fs = getActionFileSystem(context, actionXml);
                FileStatus status = fs.getFileStatus(hiveConfPath);
                if (status.isDir()) {
                    loadConfiguration(actionConf, fs, new Path(hiveConfPath, HIVE_DEFAULT_CONF));
                    loadConfiguration(actionConf, fs, new Path(hiveConfPath, HIVE_SITE_CONF));
                }
            }

            e = actionXml.getChild("configuration", ns);
            if (e != null) {
                String strConf = XmlUtils.prettyPrint(e).toString();
                XConfiguration inlineConf = new XConfiguration(new StringReader(strConf));
                checkForDisallowedProps(inlineConf, "inline configuration");
                XConfiguration.copy(inlineConf, actionConf);
            }

            String script = actionXml.getChild("script", ns).getTextTrim();
            String scriptName = new Path(script).getName();

            List<Element> params = actionXml.getChildren("param", ns);
            String[] strParams = new String[params.size()];
            for (int i = 0; i < params.size(); i++) {
                strParams[i] = params.get(i).getTextTrim();
            }

            HiveMain.setHiveScript(actionConf, scriptName, strParams);
            return actionConf;
        } catch (IOException e) {
            throw convertException(e);
        }
    }

    private void loadConfiguration(Configuration baseConf, FileSystem fs, Path confPath)
    throws ActionExecutorException {
        try {
            if (!fs.exists(confPath) || !fs.isFile(confPath)) {
                return;
            }
            Configuration newConf = new XConfiguration(fs.open(confPath));
            checkForDisallowedProps(newConf, confPath.getName());
            XConfiguration.copy(newConf, baseConf);
        } catch (IOException ex) {
            throw convertException(ex);
        }
    }

    @Override
    protected boolean getCaptureOutput(WorkflowAction action) throws JDOMException {
        return true;
    }

}
