/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.oozie.action.hadoop;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.hive.cli.CliDriver;

public class HiveMain extends LauncherMain {

    public static final String HIVE_L4J_PROPS = "hive-log4j.properties";
    public static final String HIVE_EXEC_L4J_PROPS = "hive-exec-log4j.properties";
    public static final String HIVE_DEFAULT_CONF = "hive-default.xml";
    public static final String HIVE_SITE_CONF = "hive-site.xml";
    public static final String HIVE_SCRIPT = "oozie.hive.script";
    public static final String HIVE_PARAMS = "oozie.hive.params";

    public static void main(String[] args) throws Exception {
        run(HiveMain.class, args);
    }

    private Configuration initActionConf() {
        // Loading action conf prepared by Oozie
        Configuration hiveConf = new Configuration(false);

        String actionXml = System.getProperty("oozie.action.conf.xml");

        if (actionXml == null) {
            throw new RuntimeException("Missing Java System Property [oozie.action.conf.xml]");
        }
        if (!new File(actionXml).exists()) {
            throw new RuntimeException("Action Configuration XML file ["
                    + actionXml + "] does not exist");
        } else {
            System.out.println("Using action configuration file " + actionXml);
        }

        hiveConf.addResource(new Path("file:///", actionXml));

        // Propagate delegation related props from launcher job to Hive job
        if (System.getenv("HADOOP_TOKEN_FILE_LOCATION") != null) {
            hiveConf.set("mapreduce.job.credentials.binary",
                    System.getenv("HADOOP_TOKEN_FILE_LOCATION"));
            System.out.println("------------------------");
            System.out.println("Setting env property for mapreduce.job.credentials.binary to:"
                    + System.getenv("HADOOP_TOKEN_FILE_LOCATION"));
            System.out.println("------------------------");
            System.setProperty("mapreduce.job.credentials.binary",
                    System.getenv("HADOOP_TOKEN_FILE_LOCATION"));
        } else {
            System.out.println("Non-Kerberos execution");
        }

        // Have to explicitly unset this property or Hive will not set it.
        hiveConf.set("mapred.job.name", "");

        // See https://issues.apache.org/jira/browse/HIVE-1411
        hiveConf.set("datanucleus.plugin.pluginRegistryBundleCheck", "LOG");

        return hiveConf;
    }

    protected void run(String[] args) throws Exception {
        System.out.println();
        System.out.println("Oozie Hive action configuration");
        System.out.println("=================================================================");

        Configuration hiveConf = initActionConf();

        // Write the action configuration out to hive-site.xml
        OutputStream os = new FileOutputStream(HIVE_SITE_CONF);
        hiveConf.writeXml(os);
        os.close();

        System.out.println();
        System.out.println("Hive Configuration Properties:");
        System.out.println("------------------------");
        for (Entry<String, String> entry : hiveConf) {
            System.out.println(entry.getKey() + "=" + entry.getValue());
        }
        System.out.flush();
        System.out.println("------------------------");
        System.out.println();

        List<String> arguments = new ArrayList<String>();
        String scriptPath = hiveConf.get(HIVE_SCRIPT);

        if (scriptPath == null) {
            throw new RuntimeException("Action Configuration does not have [oozie.hive.script] property");
        }

        if (!new File(scriptPath).exists()) {
            throw new RuntimeException("Hive script file [" + scriptPath + "] does not exist");
        }

        //
        // Prepare the Hive Script
        //
        System.out.println("Hive script [" + scriptPath + "] content: ");
        System.out.println("------------------------");
        System.out.println(readStringFromFile(scriptPath));
        System.out.println("------------------------");
        System.out.println();

        Map<String, String> varMap = new HashMap<String, String>();

        for (String param : MapReduceMain.getStrings(hiveConf, HIVE_PARAMS)) {
            System.out.println("Param Expression: '" + param + "'");

            int idx = param.indexOf('=');
            if (idx == -1) {
                throw new RuntimeException("Parameter expression must contain an assignment: "
                        + param);
            } else if (idx == 0) {
                throw new RuntimeException("Parameter value not specified: " + param);
            }
            String var = param.substring(0, idx);
            String val = param.substring(idx + 1, param.length());
            System.out.println("var = " + var + ", val = " + val);
            varMap.put(var, val);
        }

        if (varMap.size() > 0) {
            System.out.println("Substituting parameter values...");
            String subScript = substituteVars(varMap, readStringFromFile(scriptPath));
            scriptPath = scriptPath + ".sub";
            writeStringToFile(scriptPath, subScript);

            System.out.println("Substituted script [" + scriptPath + "] content: ");
            System.out.println("------------------------");
            System.out.println(readStringFromFile(scriptPath));
            System.out.println("------------------------");
            System.out.println();
        }

        arguments.add("-f");
        arguments.add(scriptPath);


        System.out.println("Hive command arguments :");
        for (String arg : arguments) {
            System.out.println("             " + arg);
        }

        System.out.println("=================================================================");
        System.out.println();
        System.out.println(">>> Invoking Hive command line now >>>");
        System.out.println();
        System.out.flush();

        runHive(arguments.toArray(new String[arguments.size()]));

        System.out.println("\n<<< Invocation of Hive command completed <<<\n");
    }

    private void runHive(String[] args) throws Exception {
        CliDriver.main(args);
    }

    public static void setHiveScript(Configuration conf, String script, String[] params) {
        conf.set(HIVE_SCRIPT, script);
        MapReduceMain.setStrings(conf, HIVE_PARAMS, params);
    }

    private static String readStringFromFile(String filePath) throws IOException {
        String line;
        BufferedReader br = new BufferedReader(new FileReader(filePath));
        StringBuilder sb = new StringBuilder();
        String sep = System.getProperty("line.separator");
        while ((line = br.readLine()) != null) {
            sb.append(line).append(sep);
        }
        return sb.toString();
     }

    private static void writeStringToFile(String filePath, String str) throws IOException {
        BufferedWriter out = new BufferedWriter(new FileWriter(filePath));
        out.write(str);
        out.close();
    }


    // Ripped from oahc.Configuration with slight modifications.
    // Note:
    //   Rather than break at a small pattern limit, we need Hive action to
    //   allow large substitution limit but break if there is cycle
    //   in substitution.
    private static Pattern varPat = Pattern.compile("\\$\\{[^\\}\\$\u0020]+\\}");
    static int MAX_VARIABLE_SUBST_LIMIT = 10000;

    static String substituteVars(Map<String, String> varMap, String expr) {
        Matcher match = varPat.matcher("");
        String eval = expr;
        Set<String> evalSet = new HashSet<String>();
        for(int s = 0; s < MAX_VARIABLE_SUBST_LIMIT; s++) {
            if (evalSet.contains(eval)) {
                //Cyclic resolution pattern detected. Return current expression.
                return eval;
            }
            evalSet.add(eval);
            match.reset(eval);
            if (!match.find()) {
                return eval;
            }
            String var = match.group();
            var = var.substring(2, var.length()-1); // remove ${ .. }
            String val = varMap.get(var);
            if (null == val) {
                return eval; // return literal ${var}: var is unbound
            }
            // substitute
            eval = eval.substring(0, match.start())+val+eval.substring(match.end());
        }
        throw new RuntimeException(
                "Maximum variable substitution limit exceeded: "
                + MAX_VARIABLE_SUBST_LIMIT + " " + expr);
    }
}
